@@ -99,7 +99,7 @@ enum {
 
 /* PCM interface */
 
-#define SNDRV_PCM_VERSION		SNDRV_PROTOCOL_VERSION(2, 0, 9)
+#define SNDRV_PCM_VERSION		SNDRV_PROTOCOL_VERSION(2, 0, 12)
 
 typedef unsigned long snd_pcm_uframes_t;
 typedef long snd_pcm_sframes_t;
@@ -133,7 +133,7 @@ typedef enum _snd_pcm_access {
 	SND_PCM_ACCESS_LAST = SND_PCM_ACCESS_RW_NONINTERLEAVED,
 } snd_pcm_access_t;
 
-typedef enum _sndrv_pcm_format {
+typedef enum _snd_pcm_format {
 	SND_PCM_FORMAT_UNKNOWN = -1,
 	SND_PCM_FORMAT_S8 = 0,	/* signed */
 	SND_PCM_FORMAT_U8,	/* unsigned */
@@ -173,7 +173,16 @@ typedef enum _sndrv_pcm_format {
 	SND_PCM_FORMAT_S18_3BE,
 	SND_PCM_FORMAT_U18_3LE,
 	SND_PCM_FORMAT_U18_3BE,
-	SND_PCM_FORMAT_LAST = SND_PCM_FORMAT_U18_3BE,
+	SND_PCM_FORMAT_G723_24,
+	SND_PCM_FORMAT_G723_24_1B,
+	SND_PCM_FORMAT_G723_40,
+	SND_PCM_FORMAT_G723_40_1B,
+	SND_PCM_FORMAT_DSD_U8,
+	SND_PCM_FORMAT_DSD_U16_LE,
+	SND_PCM_FORMAT_DSD_U32_LE,
+	SND_PCM_FORMAT_DSD_U16_BE,
+	SND_PCM_FORMAT_DSD_U32_BE,
+	SND_PCM_FORMAT_LAST = SND_PCM_FORMAT_DSD_U32_BE,
 
 #ifdef SND_LITTLE_ENDIAN
 	SND_PCM_FORMAT_S16 = SND_PCM_FORMAT_S16_LE,
@@ -219,6 +228,7 @@ typedef enum _snd_pcm_subformat {
 #define SNDRV_PCM_INFO_JOINT_DUPLEX	0x00200000
 #define SNDRV_PCM_INFO_SYNC_START	0x00400000
 #define SNDRV_PCM_INFO_NO_PERIOD_WAKEUP	0x00800000
+#define SNDRV_PCM_INFO_HAS_WALL_CLOCK   0x01000000
 
 typedef enum _snd_pcm_state {
 	SND_PCM_STATE_OPEN = 0,
@@ -327,6 +337,13 @@ typedef enum _snd_pcm_tstamp {
 	SND_PCM_TSTAMP_LAST = SND_PCM_TSTAMP_MMAP
 } snd_pcm_tstamp_t;
 
+typedef enum _snd_pcm_tstamp_type {
+	SND_PCM_TSTAMP_TYPE_GETTIMEOFDAY = 0,
+	SND_PCM_TSTAMP_TYPE_MONOTONIC,
+	SND_PCM_TSTAMP_TYPE_MONOTONIC_RAW,
+	SND_PCM_TSTAMP_TYPE_LAST = SND_PCM_TSTAMP_TYPE_MONOTONIC_RAW,
+} snd_pcm_tstamp_type_t;
+
 typedef struct snd_pcm_sw_params {
 	int tstamp_mode;
 	unsigned int period_step;
@@ -338,7 +355,9 @@ typedef struct snd_pcm_sw_params {
 	snd_pcm_uframes_t silence_threshold;
 	snd_pcm_uframes_t silence_size;
 	snd_pcm_uframes_t boundary;
-	unsigned char reserved[60];
+	unsigned int tstamp_type;
+	int pads;
+	unsigned char reserved[52];
 	unsigned int period_event;
 } snd_pcm_sw_params_t;
 
@@ -361,7 +380,8 @@ typedef struct snd_pcm_status {
 	snd_pcm_uframes_t avail_max;
 	snd_pcm_uframes_t overrange;
 	int suspended_state;
-	unsigned char reserved[60];
+	struct timespec audio_tstamp;
+	unsigned char reserved[60 - sizeof(struct timespec)];
 } snd_pcm_status_t;
 
 struct snd_pcm_mmap_status {
@@ -370,6 +390,7 @@ struct snd_pcm_mmap_status {
 	snd_pcm_uframes_t hw_ptr;
 	struct timespec tstamp;
 	int suspended_state;
+	struct timespec audio_tstamp;
 };
 
 struct snd_pcm_mmap_control {
@@ -405,13 +426,6 @@ struct snd_xfern {
 	snd_pcm_uframes_t frames;
 };
 
-
-enum {
-        SNDRV_PCM_TSTAMP_TYPE_GETTIMEOFDAY = 0,
-        SNDRV_PCM_TSTAMP_TYPE_MONOTONIC,
-        SNDRV_PCM_TSTAMP_TYPE_LAST = SNDRV_PCM_TSTAMP_TYPE_MONOTONIC,
-};
-
 enum {
 	SNDRV_PCM_IOCTL_PVERSION = _IOR('A', 0x00, int),
 	SNDRV_PCM_IOCTL_INFO = _IOR('A', 0x01, snd_pcm_info_t),
@@ -447,6 +461,74 @@ enum {
 /* Trick to make alsa-lib/acinclude.m4 happy */
 #define SNDRV_PCM_IOCTL_REWIND SNDRV_PCM_IOCTL_REWIND
 
+#if SALSA_HAS_CHMAP_SUPPORT
+#define SND_CHMAP_API_VERSION	((1 << 16) | (0 << 8) | 1)
+
+enum snd_pcm_chmap_type {
+	SND_CHMAP_TYPE_NONE = 0,
+	SND_CHMAP_TYPE_FIXED,
+	SND_CHMAP_TYPE_VAR,
+	SND_CHMAP_TYPE_PAIRED,
+	SND_CHMAP_TYPE_LAST = SND_CHMAP_TYPE_PAIRED,
+};
+
+enum snd_pcm_chmap_position {
+	SND_CHMAP_UNKNOWN = 0,
+	SND_CHMAP_NA,
+	SND_CHMAP_MONO,
+	SND_CHMAP_FL,
+	SND_CHMAP_FR,
+	SND_CHMAP_RL,
+	SND_CHMAP_RR,
+	SND_CHMAP_FC,
+	SND_CHMAP_LFE,
+	SND_CHMAP_SL,
+	SND_CHMAP_SR,
+	SND_CHMAP_RC,
+	SND_CHMAP_FLC,
+	SND_CHMAP_FRC,
+	SND_CHMAP_RLC,
+	SND_CHMAP_RRC,
+	SND_CHMAP_FLW,
+	SND_CHMAP_FRW,
+	SND_CHMAP_FLH,
+	SND_CHMAP_FCH,
+	SND_CHMAP_FRH,
+	SND_CHMAP_TC,
+	SND_CHMAP_TFL,
+	SND_CHMAP_TFR,
+	SND_CHMAP_TFC,
+	SND_CHMAP_TRL,
+	SND_CHMAP_TRR,
+	SND_CHMAP_TRC,
+	SND_CHMAP_TFLC,
+	SND_CHMAP_TFRC,
+	SND_CHMAP_TSL,
+	SND_CHMAP_TSR,
+	SND_CHMAP_LLFE,
+	SND_CHMAP_RLFE,
+	SND_CHMAP_BC,
+	SND_CHMAP_BLC,
+	SND_CHMAP_BRC,
+	SND_CHMAP_LAST = SND_CHMAP_BRC,
+};
+
+#define SND_CHMAP_POSITION_MASK		0xffff
+
+#define SND_CHMAP_PHASE_INVERSE		(0x01 << 16)
+#define SND_CHMAP_DRIVER_SPEC		(0x02 << 16)
+
+typedef struct snd_pcm_chmap {
+	unsigned int channels;
+	unsigned int pos[0];
+} snd_pcm_chmap_t;
+
+typedef struct snd_pcm_chmap_query {
+	enum snd_pcm_chmap_type type;
+	snd_pcm_chmap_t map;
+} snd_pcm_chmap_query_t;
+#endif /* SALSA_HAS_CHMAP_SUPPORT */
+
 /* RAW MIDI inteface */
 
 #define SNDRV_RAWMIDI_VERSION		SNDRV_PROTOCOL_VERSION(2, 0, 0)
@@ -866,6 +948,9 @@ typedef struct snd_ctl_event {
 #define SND_CTL_TLVT_DB_RANGE		3
 #define SND_CTL_TLVT_DB_MINMAX		4
 #define SND_CTL_TLVT_DB_MINMAX_MUTE	5
+#define SND_CTL_TLVT_CHMAP_FIXED	0x101	/* fixed channel position */
+#define SND_CTL_TLVT_CHMAP_VAR	0x102	/* channels freely swappable */
+#define SND_CTL_TLVT_CHMAP_PAIRED	0x103	/* pair-wise swappable */
 
 /* Mute state */
 #define SND_CTL_TLV_DB_GAIN_MUTE	-9999999
