@@ -39,14 +39,21 @@ extern "C" {
 
 #include "recipe.h"
 #include "asoundef.h"
-#include "version.h"
 #include "global.h"
+#include "version.h"
 #include "input.h"
 #include "output.h"
 #include "error.h"
 #include "control.h"
 #include "pcm.h"
 #include "mixer.h"
+#include "rawmidi.h"
+#include "hwdep.h"
+#include "timer.h"
+#include "conf.h"
+#include "seq_event.h"
+#include "seq.h"
+#include "seqmid.h"
 
 #ifdef __cplusplus
 }
