@@ -630,40 +630,35 @@ static int tlv_from_dB_range(unsigned int *tlv, long rangemin, long rangemax,
 			     long db_gain, long *value, int xdir)
 {
 	unsigned int pos, len;
-	long dbmin, dbmax, prev_rangemax;
+	long dbmin, dbmax, prev_submax;
 
 	len = int_index(tlv[1]);
-	if (len > MAX_TLV_RANGE_SIZE)
+	if (len < 6 || len > MAX_TLV_RANGE_SIZE)
 		return -EINVAL;
-	if (snd_tlv_get_dB_range(tlv, rangemin, rangemax, &dbmin, &dbmax))
-		return -EINVAL;
-	if (db_gain <= dbmin) {
-		*value = rangemin;
-		return 0;
-	} else if (db_gain >= dbmax) {
-		*value = rangemax;
-		return 0;
-	}
 	pos = 2;
-	prev_rangemax = 0;
+	prev_submax = 0;
 	while (pos + 4 <= len) {
-		rangemin = (int)tlv[pos++];
-		rangemax = (int)tlv[pos++];
-		if (!snd_tlv_get_dB_range(tlv + pos, rangemin, rangemax,
+		long submin = (int)tlv[pos++];
+		long submax = (int)tlv[pos++];
+		if (rangemax < submax)
+			submax = rangemax;
+		if (!snd_tlv_get_dB_range(tlv + pos + 2, submin, submax,
 					  &dbmin, &dbmax) &&
 		    db_gain >= dbmin && db_gain <= dbmax)
-			return snd_tlv_convert_from_dB(tlv + pos,
-						       rangemin, rangemax,
+			return snd_tlv_convert_from_dB(tlv + pos + 2,
+						       submin, submax,
 						       db_gain, value, xdir);
 		else if (db_gain < dbmin) {
-			*value = xdir ? rangemin : prev_rangemax;
+			*value = xdir > 0 || pos == 2 ? submin : prev_submax;
 			return 0;
 		}
-		prev_rangemax = rangemax;
-		pos++;
-		pos += int_index(tlv[pos]) + 1;
+		prev_submax = submax;
+		if (rangemax == submax)
+			break;
+		pos += int_index(tlv[pos + 3]) + 4;
 	}
-	return -EINVAL;
+	*value = prev_submax;
+	return 0;
 }
 
 static void _tlv_from_dB_minmax(long rangemin, long rangemax,
@@ -768,4 +763,188 @@ int snd_tlv_convert_from_dB(unsigned int *tlv, long rangemin, long rangemax,
 	return -EINVAL;
 }
 
+#define TEMP_TLV_SIZE		4096
+struct tlv_info {
+	long minval, maxval;
+	unsigned int *tlv;
+	unsigned int buf[TEMP_TLV_SIZE];
+};
+
+static int get_tlv_info(snd_ctl_t *ctl, const snd_ctl_elem_id_t *id,
+			struct tlv_info *rec)
+{
+	snd_ctl_elem_info_t info;
+	int err;
+
+	memzero_valgrind(&info, sizeof(info));
+	info.id = *id;
+	err = snd_ctl_elem_info(ctl, &info);
+	if (err < 0)
+		return err;
+	rec->minval = info.value.integer.min;
+	rec->maxval = info.value.integer.max;
+	return snd_tlv_parse_dB_info(rec->buf, sizeof(rec->buf), &rec->tlv);
+}
+
+int snd_ctl_get_dB_range(snd_ctl_t *ctl, const snd_ctl_elem_id_t *id,
+			 long *min, long *max)
+{
+	struct tlv_info info;
+	int err;
+
+	err = get_tlv_info(ctl, id, &info);
+	if (err < 0)
+		return err;
+	return snd_tlv_get_dB_range(info.tlv, info.minval, info.maxval,
+				    min, max);
+}
+
+int snd_ctl_convert_to_dB(snd_ctl_t *ctl, const snd_ctl_elem_id_t *id,
+			  long volume, long *db_gain)
+{
+	struct tlv_info info;
+	int err;
+
+	err = get_tlv_info(ctl, id, &info);
+	if (err < 0)
+		return err;
+	return snd_tlv_convert_to_dB(info.tlv, info.minval, info.maxval,
+				     volume, db_gain);
+}
+
+int snd_ctl_convert_from_dB(snd_ctl_t *ctl, const snd_ctl_elem_id_t *id,
+			    long db_gain, long *value, int xdir)
+{
+	struct tlv_info info;
+	int err;
+
+	err = get_tlv_info(ctl, id, &info);
+	if (err < 0)
+		return err;
+	return snd_tlv_convert_from_dB(info.tlv, info.minval, info.maxval,
+				       db_gain, value, xdir);
+}
 #endif /* TLV */
+
+#if SALSA_CTL_ASCII_PARSER
+char *snd_ctl_ascii_elem_id_get(snd_ctl_elem_id_t *id)
+{
+	char buf[256];
+	int len;
+
+	len = snprintf(buf, sizeof(buf), "numid=%u,iface=%s,name='%s'",
+		       id->numid, _snd_ctl_elem_iface_names[id->iface],
+		       id->name);
+	if (len < sizeof(buf) && id->index > 0)
+		len += snprintf(buf + len, sizeof(buf) - len,
+				",index=%d", id->index);
+	if (len < sizeof(buf) && id->device > 0)
+		len += snprintf(buf + len, sizeof(buf) - len,
+				",device=%d", id->device);
+	if (len < sizeof(buf) && id->subdevice > 0)
+		len += snprintf(buf + len, sizeof(buf) - len,
+				",subdevice=%d", id->subdevice);
+	return strdup(buf);
+}
+
+int snd_ctl_ascii_elem_id_parse(snd_ctl_elem_id_t *dst, const char *str)
+{
+	char iface[20];
+	char *p, *s = (char *)str;
+	int i;
+
+	while ((p = strsep(&s, ",")) != NULL) {
+		if (sscanf(p, "iface=%20[^,]", iface) == 1) {
+			for (i = 0; i <= SND_CTL_ELEM_IFACE_LAST; i++) {
+				if (!strcmp(iface,
+					    _snd_ctl_elem_iface_names[i])) {
+					dst->iface = i;
+					break;
+				}
+			}
+			if (i > SND_CTL_ELEM_IFACE_LAST)
+				return -EINVAL;
+			continue;
+		}
+		if (sscanf(p, "name='%42[^']", dst->name) == 1)
+			continue;
+		if (sscanf(p, "numid=%d", &dst->numid) == 1)
+			continue;
+		if (sscanf(p, "index=%d", &dst->index) == 1)
+			continue;
+		if (sscanf(p, "device=%d", &dst->device) == 1)
+			continue;
+		if (sscanf(p, "subdevice=%d", &dst->subdevice) == 1)
+			continue;
+		return -EINVAL;
+	}
+	return 0;
+}
+
+static int match_yes(const char *ptr)
+{
+	static const char * const yes[] = {
+		"on", "up", "yes", "true", NULL,
+	};
+	const char * const *p;
+
+	for (p = yes; *p; p++)
+		if (strncasecmp(ptr, *p, strlen(*p)))
+			return 1;
+	return 0;
+}
+
+static int get_enum_index(snd_ctl_t *handle, snd_ctl_elem_info_t *info,
+			  const char *ptr)
+{
+	int i;
+
+	for (i = 0; i < info->value.enumerated.items; i++) {
+		info->value.enumerated.item = i;
+		if (snd_ctl_elem_info(handle, info) < 0)
+			return -1;
+		if (!strncmp(ptr, info->value.enumerated.name,
+			     strlen(info->value.enumerated.name)))
+			return i;
+	}
+	return -1;
+}
+
+int snd_ctl_ascii_value_parse(snd_ctl_t *handle,
+			      snd_ctl_elem_value_t *dst,
+			      snd_ctl_elem_info_t *info,
+			      const char *ptr)
+{
+	int idx = 0;
+	int tmp;
+
+	for (idx = 0; idx < info->count && idx < 128 && *ptr; idx++) {
+		switch (info->type) {
+		case SND_CTL_ELEM_TYPE_BOOLEAN:
+			if (!strncasecmp(ptr, "toggle", 6))
+				tmp = !dst->value.integer.value[idx];
+			else if (match_yes(ptr) || *ptr == '1')
+				tmp = 1;
+			else
+				tmp = 0;
+			dst->value.integer.value[idx] = tmp;
+			break;
+		case SND_CTL_ELEM_TYPE_INTEGER:
+			dst->value.integer.value[idx] = strtol(ptr, NULL, 0);
+			break;
+		case SND_CTL_ELEM_TYPE_INTEGER64:
+			dst->value.integer.value[idx] = strtoll(ptr, NULL, 0);
+			break;
+		case SND_CTL_ELEM_TYPE_ENUMERATED:
+			tmp = get_enum_index(handle, info, ptr);
+			if (tmp < 0)
+				return -EINVAL;
+			dst->value.enumerated.item[idx] = tmp;
+			break;
+		}
+		while (*ptr && *ptr != ',')
+			ptr++;
+	}
+	return 0;
+}
+#endif /* SALSA_CTL_ASCII_PARSER */
