@@ -27,7 +27,12 @@ int snd_card_get_index(const char *name);
 int snd_card_get_name(int card, char **name);
 int snd_card_get_longname(int card, char **name);
 
+#if SALSA_CHECK_ABI
+int _snd_ctl_open(snd_ctl_t **ctl, const char *name, int mode,
+		  unsigned int magic);
+#else
 int snd_ctl_open(snd_ctl_t **ctl, const char *name, int mode);
+#endif
 int snd_ctl_close(snd_ctl_t *ctl);
 int snd_ctl_wait(snd_ctl_t *ctl, int timeout);
 
@@ -46,6 +51,7 @@ int snd_async_add_ctl_handler(snd_async_handler_t **handler, snd_ctl_t *ctl,
 			      void *private_data);
 #endif
 
+#if SALSA_HAS_USER_ELEM_SUPPORT
 int snd_ctl_elem_add_integer(snd_ctl_t *ctl, const snd_ctl_elem_id_t *id,
 			     unsigned int count, long min, long max, long step);
 int snd_ctl_elem_add_integer64(snd_ctl_t *ctl, const snd_ctl_elem_id_t *id,
@@ -54,8 +60,15 @@ int snd_ctl_elem_add_integer64(snd_ctl_t *ctl, const snd_ctl_elem_id_t *id,
 int snd_ctl_elem_add_boolean(snd_ctl_t *ctl, const snd_ctl_elem_id_t *id,
 			     unsigned int count);
 int snd_ctl_elem_add_iec958(snd_ctl_t *ctl, const snd_ctl_elem_id_t *id);
+int snd_ctl_elem_add_enumerated(snd_ctl_t *ctl, const snd_ctl_elem_id_t *id,
+				unsigned int count, unsigned int items,
+				const char *const names[]);
+#endif
 
 #if SALSA_HAS_TLV_SUPPORT
+int snd_tlv_parse_dB_info(unsigned int *tlv,
+			  unsigned int tlv_size,
+			  unsigned int **db_tlvp);
 int snd_tlv_get_dB_range(unsigned int *tlv, long rangemin, long rangemax,
 			 long *min, long *max);
 int snd_tlv_convert_to_dB(unsigned int *tlv, long rangemin, long rangemax,
