@@ -478,12 +478,42 @@ int snd_mixer_selem_get_capture_switch(snd_mixer_elem_t *elem,
 					   channel, value);
 }
 
+extern int _snd_selem_update_volume(snd_selem_vol_item_t *str, int channel,
+				    long value);
+extern int _snd_selem_update_volume_all(snd_selem_vol_item_t *str, long value);
+
+__SALSA_EXPORT_FUNC
+int snd_mixer_selem_set_playback_volume(snd_mixer_elem_t *elem,
+					snd_mixer_selem_channel_id_t channel,
+					long value)
+{
+	return _snd_selem_update_volume(elem->items[SND_SELEM_ITEM_PVOLUME], channel, value);
+}
+
+__SALSA_EXPORT_FUNC
+int snd_mixer_selem_set_capture_volume(snd_mixer_elem_t *elem,
+				       snd_mixer_selem_channel_id_t channel,
+				       long value)
+{
+	return _snd_selem_update_volume(elem->items[SND_SELEM_ITEM_CVOLUME], channel, value);
+}
+
+__SALSA_EXPORT_FUNC
+int snd_mixer_selem_set_playback_volume_all(snd_mixer_elem_t *elem, long value)
+{
+	return _snd_selem_update_volume_all(elem->items[SND_SELEM_ITEM_PVOLUME], value);
+}
+
+__SALSA_EXPORT_FUNC
+int snd_mixer_selem_set_capture_volume_all(snd_mixer_elem_t *elem, long value)
+{
+	return _snd_selem_update_volume_all(elem->items[SND_SELEM_ITEM_CVOLUME], value);
+}
+
 __SALSA_EXPORT_FUNC
 int snd_mixer_selem_is_enumerated(snd_mixer_elem_t *elem)
 {
-	snd_selem_enum_item_t *eitem =
-		(snd_selem_enum_item_t *) elem->items[SND_SELEM_ITEM_ENUM];
-	return eitem != NULL;
+	return elem->items[SND_SELEM_ITEM_ENUM] != NULL;
 }
 
 __SALSA_EXPORT_FUNC
@@ -501,8 +531,7 @@ int snd_mixer_selem_is_enum_capture(snd_mixer_elem_t *elem)
 __SALSA_EXPORT_FUNC
 int snd_mixer_selem_get_enum_items(snd_mixer_elem_t *elem)
 {
-	snd_selem_enum_item_t *eitem =
-		(snd_selem_enum_item_t *) elem->items[SND_SELEM_ITEM_ENUM];
+	snd_selem_enum_item_t *eitem = elem->items[SND_SELEM_ITEM_ENUM];
 	if (!eitem)
 		return -EINVAL;
 	return eitem->items;
@@ -513,8 +542,7 @@ int snd_mixer_selem_get_enum_item(snd_mixer_elem_t *elem,
 				  snd_mixer_selem_channel_id_t channel,
 				  unsigned int *itemp)
 {
-	snd_selem_enum_item_t *eitem =
-		(snd_selem_enum_item_t *) elem->items[SND_SELEM_ITEM_ENUM];
+	snd_selem_enum_item_t *eitem = elem->items[SND_SELEM_ITEM_ENUM];
 	if (!eitem || (unsigned int)channel >= eitem->head.channels)
 		return -EINVAL;
 	*itemp =  eitem->item[channel];
@@ -562,8 +590,9 @@ extern int _snd_selem_vol_get_dB_range(snd_selem_vol_item_t *item,
 extern int _snd_selem_vol_get_dB(snd_selem_vol_item_t *item, int channel,
 				 long *value);
 extern int _snd_selem_vol_set_dB(snd_selem_vol_item_t *item,
-				 snd_mixer_selem_channel_id_t channel,
-				 long db_gain, int xdir);
+				 int channel, long db_gain, int xdir);
+extern int _snd_selem_vol_set_dB_all(snd_selem_vol_item_t *item,
+				     long db_gain, int xdir);
 extern int _snd_selem_ask_vol_dB(snd_selem_vol_item_t *item, long value,
 				 long *dBvalue);
 extern int _snd_selem_ask_dB_vol(snd_selem_vol_item_t *item, long dBvalue,
@@ -573,7 +602,7 @@ __SALSA_EXPORT_FUNC
 int snd_mixer_selem_get_playback_dB_range(snd_mixer_elem_t *elem,
 					  long *min, long *max)
 {
-	return _snd_selem_vol_get_dB_range((snd_selem_vol_item_t *)elem->items[SND_SELEM_ITEM_PVOLUME],
+	return _snd_selem_vol_get_dB_range(elem->items[SND_SELEM_ITEM_PVOLUME],
 					   min, max);
 }
 
@@ -582,7 +611,7 @@ int snd_mixer_selem_get_playback_dB(snd_mixer_elem_t *elem,
 				    snd_mixer_selem_channel_id_t channel,
 				    long *value)
 {
-	return _snd_selem_vol_get_dB((snd_selem_vol_item_t *)elem->items[SND_SELEM_ITEM_PVOLUME],
+	return _snd_selem_vol_get_dB(elem->items[SND_SELEM_ITEM_PVOLUME],
 				     channel, value);
 }
 
@@ -590,7 +619,7 @@ __SALSA_EXPORT_FUNC
 int snd_mixer_selem_ask_playback_vol_dB(snd_mixer_elem_t *elem, long value,
 					long *dBvalue)
 {
-	return _snd_selem_ask_vol_dB((snd_selem_vol_item_t *)elem->items[SND_SELEM_ITEM_PVOLUME],
+	return _snd_selem_ask_vol_dB(elem->items[SND_SELEM_ITEM_PVOLUME],
 				     value, dBvalue);
 }
 
@@ -598,7 +627,7 @@ __SALSA_EXPORT_FUNC
 int snd_mixer_selem_ask_playback_dB_vol(snd_mixer_elem_t *elem, long dBvalue,
 					int dir, long *value)
 {
-	return _snd_selem_ask_dB_vol((snd_selem_vol_item_t *)elem->items[SND_SELEM_ITEM_PVOLUME],
+	return _snd_selem_ask_dB_vol(elem->items[SND_SELEM_ITEM_PVOLUME],
 				     dBvalue, value, dir);
 }
 
@@ -607,35 +636,22 @@ int snd_mixer_selem_set_playback_dB(snd_mixer_elem_t *elem,
 				    snd_mixer_selem_channel_id_t channel,
 				    long value, int dir)
 {
-	return _snd_selem_vol_set_dB((snd_selem_vol_item_t *)elem->items[SND_SELEM_ITEM_PVOLUME],
-				     channel, value, dir);
+	return _snd_selem_vol_set_dB(elem->items[SND_SELEM_ITEM_PVOLUME],
+				     (int)channel, value, dir);
 }
 
 __SALSA_EXPORT_FUNC
 int snd_mixer_selem_set_playback_dB_all(snd_mixer_elem_t *elem, long value,
 					int dir)
 {
-	unsigned int i;
-	int err;
-	snd_selem_vol_item_t *vol =
-		(snd_selem_vol_item_t *) elem->items[SND_SELEM_ITEM_PVOLUME];
-	if (!vol)
-		return -EINVAL;
-	for (i = 0; i < vol->head.channels; i++) {
-		err = _snd_selem_vol_set_dB(vol,
-					    (snd_mixer_selem_channel_id_t)i,
-					    value, dir);
-		if (err < 0)
-			return err;
-	}
-	return 0;
+	return _snd_selem_vol_set_dB_all(elem->items[SND_SELEM_ITEM_PVOLUME], value, dir);
 }
 
 __SALSA_EXPORT_FUNC
 int snd_mixer_selem_get_capture_dB_range(snd_mixer_elem_t *elem,
 					 long *min, long *max)
 {
-	return _snd_selem_vol_get_dB_range((snd_selem_vol_item_t *)elem->items[SND_SELEM_ITEM_CVOLUME],
+	return _snd_selem_vol_get_dB_range(elem->items[SND_SELEM_ITEM_CVOLUME],
 					   min, max);
 }
 
@@ -644,7 +660,7 @@ int snd_mixer_selem_get_capture_dB(snd_mixer_elem_t *elem,
 				   snd_mixer_selem_channel_id_t channel,
 				   long *value)
 {
-	return _snd_selem_vol_get_dB((snd_selem_vol_item_t *)elem->items[SND_SELEM_ITEM_CVOLUME],
+	return _snd_selem_vol_get_dB(elem->items[SND_SELEM_ITEM_CVOLUME],
 				     channel, value);
 }
 
@@ -652,7 +668,7 @@ __SALSA_EXPORT_FUNC
 int snd_mixer_selem_ask_capture_vol_dB(snd_mixer_elem_t *elem, long value,
 				       long *dBvalue)
 {
-	return _snd_selem_ask_vol_dB((snd_selem_vol_item_t *)elem->items[SND_SELEM_ITEM_CVOLUME],
+	return _snd_selem_ask_vol_dB(elem->items[SND_SELEM_ITEM_CVOLUME],
 				     value, dBvalue);
 }
 
@@ -660,7 +676,7 @@ __SALSA_EXPORT_FUNC
 int snd_mixer_selem_ask_capture_dB_vol(snd_mixer_elem_t *elem, long dBvalue,
 				       int dir, long *value)
 {
-	return _snd_selem_ask_dB_vol((snd_selem_vol_item_t *)elem->items[SND_SELEM_ITEM_CVOLUME],
+	return _snd_selem_ask_dB_vol(elem->items[SND_SELEM_ITEM_CVOLUME],
 				     dBvalue, value, dir);
 }
 
@@ -669,7 +685,7 @@ int snd_mixer_selem_set_capture_dB(snd_mixer_elem_t *elem,
 				   snd_mixer_selem_channel_id_t channel,
 				   long value, int dir)
 {
-	return _snd_selem_vol_set_dB((snd_selem_vol_item_t *)elem->items[SND_SELEM_ITEM_CVOLUME],
+	return _snd_selem_vol_set_dB(elem->items[SND_SELEM_ITEM_CVOLUME],
 				     channel, value, dir);
 }
 
@@ -677,20 +693,7 @@ __SALSA_EXPORT_FUNC
 int snd_mixer_selem_set_capture_dB_all(snd_mixer_elem_t *elem, long value,
 				       int dir)
 {
-	unsigned int i;
-	int err;
-	snd_selem_vol_item_t *vol =
-		(snd_selem_vol_item_t *) elem->items[SND_SELEM_ITEM_CVOLUME];
-	if (!vol)
-		return -EINVAL;
-	for (i = 0; i < vol->head.channels; i++) {
-		err = _snd_selem_vol_set_dB(vol,
-					    (snd_mixer_selem_channel_id_t)i,
-					    value, dir);
-		if (err < 0)
-			return err;
-	}
-	return 0;
+	return _snd_selem_vol_set_dB_all(elem->items[SND_SELEM_ITEM_CVOLUME], value, dir);
 }
 
 #else /* SALSA_HAS_TLV_SUPPORT */
