@@ -462,6 +462,12 @@ int snd_pcm_hw_params_can_sync_start(const snd_pcm_hw_params_t *params)
 	return !!(params->info & SNDRV_PCM_INFO_SYNC_START);
 }
 
+__SALSA_EXPORT_FUNC
+int snd_pcm_hw_params_supports_audio_wallclock_ts(const snd_pcm_hw_params_t *params)
+{
+	return !!(params->info & SNDRV_PCM_INFO_HAS_WALL_CLOCK);
+}
+
 __SALSA_EXPORT_FUNC
 int snd_pcm_hw_params_get_rate_numden(const snd_pcm_hw_params_t *params,
 				      unsigned int *rate_num,
@@ -1807,6 +1813,22 @@ int snd_pcm_sw_params_get_tstamp_mode(const snd_pcm_sw_params_t *params,
 	return 0;
 }
 
+__SALSA_EXPORT_FUNC
+int snd_pcm_sw_params_set_tstamp_type(snd_pcm_t *pcm, snd_pcm_sw_params_t *params,
+				      snd_pcm_tstamp_type_t val)
+{
+	params->tstamp_type = val;
+	return 0;
+}
+
+__SALSA_EXPORT_FUNC
+int snd_pcm_sw_params_get_tstamp_type(const snd_pcm_sw_params_t *params,
+				      snd_pcm_tstamp_type_t *val)
+{
+	*val = (snd_pcm_tstamp_type_t) params->tstamp_type;
+	return 0;
+}
+
 __SALSA_EXPORT_FUNC
 __SALSA_DEPRECATED
 int snd_pcm_sw_params_set_sleep_min(snd_pcm_t *pcm, snd_pcm_sw_params_t *params,
@@ -1985,6 +2007,12 @@ void snd_pcm_status_get_htstamp(const snd_pcm_status_t *obj, snd_htimestamp_t *p
 	*ptr = obj->tstamp;
 }
 
+__SALSA_EXPORT_FUNC
+void snd_pcm_status_get_audio_htstamp(const snd_pcm_status_t *obj, snd_htimestamp_t *ptr)
+{
+	*ptr = obj->audio_tstamp;
+}
+
 __SALSA_EXPORT_FUNC
 snd_pcm_sframes_t snd_pcm_status_get_delay(const snd_pcm_status_t *obj)
 {
@@ -2323,4 +2351,35 @@ u_int8_t snd_pcm_format_silence(snd_pcm_format_t format)
 	return (u_int8_t)snd_pcm_format_silence_64(format);
 }
 
+#if SALSA_HAS_CHMAP_SUPPORT
+__SALSA_EXPORT_FUNC
+snd_pcm_chmap_query_t **snd_pcm_query_chmaps(snd_pcm_t *pcm)
+{
+	return snd_pcm_query_chmaps_from_hw(pcm->card, pcm->device,
+					    pcm->subdevice, pcm->stream);
+}
+
+extern const char *_snd_chmap_type_names[];
+extern const char *_snd_chmap_names[];
+extern const char *_snd_chmap_long_names[];
+
+__SALSA_EXPORT_FUNC
+const char *snd_pcm_chmap_type_name(enum snd_pcm_chmap_type val)
+{
+	return _snd_chmap_type_names[val];
+}
+
+__SALSA_EXPORT_FUNC
+const char *snd_pcm_chmap_name(enum snd_pcm_chmap_position val)
+{
+	return _snd_chmap_names[val];
+}
+
+__SALSA_EXPORT_FUNC
+const char *snd_pcm_chmap_long_name(enum snd_pcm_chmap_position val)
+{
+	return _snd_chmap_long_names[val];
+}
+#endif /* SALSA_HAS_CHMAP_SUPPORT */
+
 #endif /* __ALSA_PCM_MACROS_H */
