@@ -347,6 +347,7 @@ static int snd_pcm_sw_params_default(snd_pcm_t *pcm,
 				     snd_pcm_sw_params_t *params)
 {
 	params->tstamp_mode = SND_PCM_TSTAMP_NONE;
+	params->tstamp_type = pcm->sw_params.tstamp_type;
 	params->period_step = 1;
 	params->sleep_min = 0;
 	params->avail_min = pcm->period_size;
@@ -436,6 +437,7 @@ static void _snd_pcm_hw_params_any(snd_pcm_hw_params_t *params)
 	params->rmask = ~0U;
 	params->cmask = 0;
 	params->info = ~0U;
+	params->flags = SNDRV_PCM_HW_PARAMS_NORESAMPLE;
 }
 
 int snd_pcm_hw_params_any(snd_pcm_t *pcm, snd_pcm_hw_params_t *params)
@@ -590,11 +592,9 @@ int _snd_pcm_hw_param_set_max(snd_pcm_t *pcm, snd_pcm_hw_params_t *params,
 
 	save = *params;
 	err = hw_param_set_max(params, var, *val, dir ? *dir : 0);
-	if (!err) {
-		err = hw_param_update_var(pcm, params, var, err);
-		if (!err)
-			return _snd_pcm_hw_param_get_max(params, var, val, dir);
-	}
+	err = hw_param_update_var(pcm, params, var, err);
+	if (!err)
+		return _snd_pcm_hw_param_get_max(params, var, val, dir);
 	*params = save;
 	return err;
 }
@@ -683,7 +683,7 @@ int _snd_pcm_hw_param_set(snd_pcm_t *pcm, snd_pcm_hw_params_t *params,
 	snd_pcm_hw_params_t save = *params;
 	int err = hw_param_set(params, var, val, dir);
 	err = hw_param_update_var(pcm, params, var, err);
-	if (err)
+	if (err < 0)
 		*params = save;
 	return err;
 }
@@ -712,7 +712,7 @@ int _snd_pcm_hw_param_set_integer(snd_pcm_t *pcm,
 	save = *params;
 	i->integer = 1;
 	err = hw_param_update_var(pcm, params, var, 1);
-	if (err)
+	if (err < 0)
 		*params = save;
 	return err;
 }
@@ -827,7 +827,7 @@ int _snd_pcm_hw_param_set_near(snd_pcm_t *pcm, snd_pcm_hw_params_t *params,
 	else if (valdir == 0)
 		maxdir = -1;
 	else {
-		valdir = 1;
+		maxdir = 1;
 		max--;
 	}
 	save = *params;
